/*
Module : INT96.H
Purpose: Interface for a 96 bit integer class
Created: PJN / 24-04-1998

Copyright (c) 1998 - 2006 by PJ Naughter.  (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code. 

*/

////////// Defines ///////////////////////

#ifndef __INT96_H__
#define __INT96_H__


#ifdef __GNUC__
  #include "assert.h"
  #define ASSERT             assert
  #define BOOL               bool
  #define TRUE               1
  #define FALSE              0
  typedef unsigned long      DWORD;
  typedef long long          i64;
  typedef unsigned long long u_i64;
#else
  typedef __int64          i64;
  typedef unsigned __int64 u_i64;
#endif



////////// Classes ///////////////////////

class CInt96
{
public:
//constructors / destructors
  CInt96();
  CInt96(unsigned short value);
  CInt96(unsigned int value);
  CInt96(unsigned long value);
  CInt96(const u_i64& value);
  CInt96(short value);
  CInt96(int value);
  CInt96(long value);
  CInt96(const i64& value);
  CInt96(const CInt96& value);

//assignment operator
  CInt96& operator=(const CInt96& value);

//arithmetic operators
  CInt96 operator+(const CInt96& value);
  CInt96 operator-(const CInt96& value);
  friend CInt96 operator-(const CInt96& value);
  CInt96& operator++();
  CInt96& operator--();
  CInt96  operator*(const CInt96& value) const;
  CInt96  operator/(const CInt96& value) const;
  CInt96  operator%(const CInt96& value) const;
  CInt96& operator*=(const CInt96& value);
  CInt96& operator/=(const CInt96& value);
  CInt96& operator+=(const CInt96& value);
  CInt96& operator-=(const CInt96& value);
  CInt96& operator%=(const CInt96& value);
  CInt96  operator~() const;

//equality operators
  int operator==(const CInt96& value) const;
  int operator!=(const CInt96& value) const;
  int operator>(const CInt96& value) const;
  int operator>=(const CInt96& value) const;
  int operator<(const CInt96& value) const;
  int operator<=(const CInt96& value) const;

//Misc operators
  CInt96  operator>>(int nShift) const;
  CInt96  operator<<(int nShift) const;
  CInt96& operator>>=(int nShift);
  CInt96& operator<<=(int nShift);
  CInt96  operator^(const CInt96& value) const;
  CInt96  operator|(const CInt96& value) const;
  CInt96  operator&(const CInt96& value) const;
  CInt96& operator^=(const CInt96& value);
  CInt96& operator|=(const CInt96& value);
  CInt96& operator&=(const CInt96& value);

//Operators to convert back to basic types
  operator int();
  operator unsigned int();
  operator i64();
  operator u_i64();

//String conversion functions
#ifdef _AFX //Note string conversion functions are currently only supported when using MFC
  CString FormatAsHex(BOOL bLeadingZeros = TRUE) const;
  CString FormatAsBinary(BOOL bLeadingZeros = TRUE) const;
  CString FormatAsDecimal() const;
  BOOL ConvertFromBinaryString(const CString& sText);
  BOOL ConvertFromHexString(const CString& sText);
  BOOL ConvertFromDecimalString(const CString& sText);
#endif

//Misc. Functions
  BOOL GetBit(int nIndex) const;
  void SetBit(int nIndex, BOOL value);
  BOOL IsZero() const;
  void Zero();
  void Negate();
  BOOL IsNegative() const;
  BOOL IsPositive() const;
  void Modulus(const CInt96& divisor, CInt96& Quotient, CInt96& Remainder) const;

//Serialization
#ifdef _AFX //Nore serialization is MFC specific
  virtual void Serialize(CArchive& ar);
#endif

protected:
  void TwosComplement();
  void InverseTwosComplement();

  //Actual member data variables
  DWORD m_MSB;
  DWORD m_CSB;
  DWORD m_LSB;
};

#endif //__INT96_H__
